# Generated by Django 4.2.6 on 2023-10-06 03:24

from django.db import migrations, models
import django.db.models.deletion
from django.utils import timezone
import django_tasks.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RegisteredTask',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dotted_path', models.CharField(max_length=80, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='DocTask',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('scheduled_at', models.DateTimeField(default=timezone.now)),
                ('completed_at', models.DateTimeField(null=True)),
                ('inputs', models.JSONField(default=dict, encoder=django_tasks.models.DefensiveJsonEncoder)),
                ('document', models.JSONField(default=list, encoder=django_tasks.models.DefensiveJsonEncoder)),
                ('registered_task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_tasks.registeredtask')),
            ],
        ),
    ]
